@@ -102,28 +102,34 @@ def docs():
 # ※※other common python commands:
 def cmd():
     print(
-        "[1] pd.concat([df1,df2]) | 'combine df'\n"
-        "[2] list(df) | 'show all column name'\n"
-        "[3] df.drop_duplicates() | 'remove duplicate rows'\n"
-        "[4] df.drop(['B', 'C'], axis=1) | 'delete columns'\n"
-        "[5] df1 = df.loc[df['parameter name']==xxx] | 'filter dataset with conditions'\n"
-        "[6] df.dropna(axis=1) | 'remove nan'\n"
-        "[7] df = df.rename(columns = {'engine serial number':'esn'}) | 'change column name'\n"
-        "[8] df2.sort_values(by=['xxx'],inplace=True) | 'sort values'\n"
-        "[9] set(df) | 'remove duplicate for one parameter'\n"
-        "[10] df['new column'] = df['existing column'] + 100 | 'add a new column' \n"
-        "[11] pip show (package name) | 'show package details'\n"
-        "[12] data.loc[(data['para1'] == x) & (data['para-3']>xx)] | 'filter multiple conditions'\n"
-        "[13] df.describe() | 'show dataframe basic statistics values'\n"
-        "[14] df.info() | 'show dataframe basic summary'\n"
-        "[15] df.iloc[0,:] | 'select first row'\n"
-        "[16] df.iloc[:,0] | 'select first column'\n"
-        "[17] df.drop([‘column1', ‘column2'], axis=1, inplace=True) | 'drop some columns'\n"
-        "[18] df_test1.sort_values(by=[‘active’], inplace=True, ascending=True/False) | 'sort values'\n"
-        "[19] df.groupby([‘x1’,’x2’]).sum(), xx.reset_index() | 'group by some value'\n"
-        "[20] df3 = df2.stack().rename_axis(('Raws','Feature name')).reset_index(name='xx') | 'stack(),unstack() rows->columns\n'"
-        "[21] random.randint(0,100) for _ in range(20) | random.choice(['type-1','type-2']) for _ in range(20) | 'build some random value'\n"
-        "[22] data.loc[(data['para1'] == x)|(data['para-3']>xx)] | 'filter multiple conditions, or-'|', and-'&''\n"
+        "[1] pd.concat([df1,df2]) | (combine df)\n"
+        "[2] list(df) | (show all column name)\n"
+        "[3] df.drop_duplicates() | (remove duplicate rows)\n"
+        "[4] df.drop(['B', 'C'], axis=1) | (delete columns)\n"
+        "[5] df1 = df.loc[df['parameter name']==xxx] | (filter dataset with conditions)\n"
+        "[6] df.dropna(axis=1) | (remove nan)\n"
+        "[7] df = df.rename(columns = {'engine serial number':'esn'}) | (change column name)\n"
+        "[8] df2.sort_values(by=['xxx'],inplace=True) | (sort values)\n"
+        "[9] set(df) | (remove duplicate for one parameter)\n"
+        "[10] df['new column'] = df['existing column'] + 100 | (add a new column) \n"
+        "[11] pip show (package name) | (show package details)\n"
+        "[12] data.loc[(data['para1'] == x) & (data['para-3']>xx)] | (filter multiple conditions)\n"
+        "[13] df.describe() | (show dataframe basic statistics values)\n"
+        "[14] df.info() | (show dataframe basic summary)\n"
+        "[15] df.iloc[0,:] | (select first row)\n"
+        "[16] df.iloc[:,0] | (select first column)\n"
+        "[17] df.drop([‘column1', ‘column2'], axis=1, inplace=True) | (drop some columns)\n"
+        "[18] df_test1.sort_values(by=[‘active’], inplace=True, ascending=True/False) | (sort values)\n"
+        "[19] df.groupby([‘x1’,’x2’]).sum(), xx.reset_index() | (group by some value)\n"
+        "[20] df3 = df2.stack().rename_axis(('Raws','Feature name')).reset_index(name='xx') | (stack(),unstack() rows->columns)\n"
+        "[21] random.randint(0,100) for _ in range(20) | random.choice(['type-1','type-2']) for _ in range(20) | (build some random value)\n"
+        "[22] data.loc[(data['para1'] == x)|(data['para-3']>xx)] | (filter multiple conditions, or-'|', and-'&')\n"
+        "[23] df=pd.read_excel('dataset.xlsx') | (pip install xlrd'first,read excel file.)\n"
+        "[24] print(isinstance('x', int)) | (judge whether x is int) \n"
+        "[25] string[:-4] | (remove last 4 words of this string)\n"
+        "[26] xx.strftime('%Y-%M-%d %H:%M:%S') | (xx is timestamp value, transfer timestamp into str) \n"
+        "[27] dftemp[j].strftime('%H:%M:%S') | (change timestamp format)\n"
+        "[28] test2 = [x for x in test if x] | (remove anything is 'empty' related)\n"
         "[xx] xx to be continue.."
     )
 
@@ -1744,21 +1750,22 @@ def line04(df, parameterx, parametery, std_method, gap, title):
 
     df_mean = df.groupby(parameterx).mean()
 
-    if std_method == 'm1':
-        df_se = df.groupby(parameterx).agg(np.std)
-    else:
-        df_se = np.std(df[parametery])
-
     plt.figure()
     plt.grid(alpha=0.3)
     x = df_mean.index
 
-    plt.plot(x, df_mean[parametery], '-o', color="white", lw=2, alpha=1, mfc='steelblue')
+    if std_method == 'm1':
+        df_se = df.groupby(parameterx).agg(np.std)
+        plt.plot(x, df_mean[parametery], '-o', color="lightblue", lw=2, alpha=1, mfc='steelblue')
+    else:
+        df_se = np.std(df[parametery])
+        plt.plot(x, df_mean[parametery], '-o', color="white", lw=2, alpha=1, mfc='steelblue')
+
     y1_temp = df_mean - df_se
     y1 = y1_temp[parametery].tolist()
     y2_temp = df_mean + df_se
     y2 = y2_temp[parametery].tolist()
-    plt.fill_between(x, y1, y2, color="steelblue")
+    plt.fill_between(x, y1, y2, color="steelblue", alpha=0.8)
 
     plt.xticks(x[::gap], [str(d) for d in x[::gap]], rotation=45)
     plt.title(title)
@@ -1767,27 +1774,67 @@ def line04(df, parameterx, parametery, std_method, gap, title):
     plt.show()
 
 
+# networks - show relationship between different entities
+def network01(df, entity1, entity2, relation, nodesize, fontsize, layout, title):
+    #   ## show all the relations
 
+    #     demodata = {
+    #     'entity1':['A','A','A','A','A','B','B','B','B','B','F','F','F','G','G','G','G','D','E','G'],
+    #     'entity2':['B','C','D','E','F','A','C','D','E','F','C','D','E','A','C','E','D','E','C','B'],
+    #     'relation':[
+    #                 'relation_x','relation_x','relation_x','relation_x','relation_x',
+    #                 'relation_x','relation_x','relation_x','relation_x','relation_x',
+    #                 'relation_x','relation_x','relation_x','relation_x','relation_x',
+    #                 'relation_x','relation_x','relation_x','relation_x','relation_x',
+    #                ]}
 
-warning_threshold = 'Below modules are not validated. update:1/1---------------------👇'
+    import networkx as nx
+    import matplotlib.pyplot as plt
+    from pyvis.network import Network
+
+    G = nx.DiGraph()
+    df = df
+
+    G.add_nodes_from(set(df[entity1]).union(set(df[entity2])))
+    for i in range(len(df)):
+        relation1 = df.iloc[i, 2]
+        G.add_edge(df.iloc[i, 0], df.iloc[i, 1], relation=relation1)
+
+    labels_info = nx.get_edge_attributes(G, relation)
+    #     print(labels_info)
+
+    plt.figure(figsize=(8, 8))
+
+    # layout: spiral | circular(circle) | sheel(concentric circle)
+    if layout == 'spiral':
+        pos = nx.spiral_layout(G)
+    elif layout == 'circular':
+        pos = nx.circular_layout(G)
+    else:
+        pos = nx.shell_layout(G)
+
+    nx.draw(G, with_labels=True, node_color='skyblue', alpha=1, node_size=nodesize, edge_cmap=plt.cm.Blues, pos=pos)
+    nx.draw_networkx_edge_labels(G, pos=pos, edge_labels=labels_info, font_color='blue', font_size=fontsize)
+    plt.title(title)
+    plt.show()
+
+    return
 
 
 
+this_is_nothing = 'Below modules are not validated. update:1/1---------------------👇'
 
-# show correlation line + error zone. with scatter plot:
 
 
-# [planning ↓]
 
 
 # Line plot (x-时间序列: 4.移动平均图 Arima  5.时间序列预测 等种种）
 # 多项式回归拟合
 # 聚类算法 DBSCAN.
-# regression model:
 # 多元线性回归：https://cloud.tencent.com/developer/article/1059976
 # 移动平均模型（ARIMA）
 # 时间序列预测图：自相关性，偏自相关图等等
 # wordcloud plot(词云图)
 # 贝叶斯统计方法-python实现
 # python自动化爬虫
-# SQL
\ No newline at end of file
+# 递归算法
\ No newline at end of file
